"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ExtensionDebuggerTransport {
    constructor(target) {
        this.responseDelay = 0.025 * 1000;
        this.target = target;
        this._sessionId = target.id;
        this.debugee = {
            tabId: target.tabId,
        };
        chrome.debugger.onEvent.addListener((source, method, params) => {
            const event = {
                method: method,
                params: params,
                sessionId: this._sessionId,
            };
            console.log('event', event);
            source.tabId === this.target.tabId ? this._emit(event) : null;
        });
        chrome.debugger.onDetach.addListener(source => {
            source.tabId === this.target.tabId ? this._closeTarget() : null;
        });
    }
    static create(tabId) {
        if (chrome.debugger) {
            const debugee = {
                tabId: tabId,
            };
            return new Promise((resolve, reject) => {
                chrome.debugger.attach(debugee, '1.3', async () => {
                    const error = chrome.runtime.lastError;
                    if (!error) {
                        const target = await this._getTargetInfo(debugee);
                        const transport = new ExtensionDebuggerTransport(target);
                        resolve(transport);
                    }
                    else {
                        reject(error);
                    }
                });
            });
        }
        else {
            throw new Error('no debugger permission');
        }
    }
    send(message) {
        const command = JSON.parse(message);
        const targetCommands = [
            'Target.getBrowserContexts',
            'Target.setDiscoverTargets',
            'Target.attachToTarget',
            'Target.activateTarget',
            'Target.closeTarget',
        ];
        if (targetCommands.includes(command.method)) {
            this._handleTargetCommand(command);
        }
        else {
            chrome.debugger.sendCommand(this.debugee, command.method, command.params, result => this._handleCommandResponse(command, result));
        }
    }
    close() {
        chrome.debugger.detach(this.debugee, () => this._closeTarget());
    }
    static _getTargetInfo(debugee) {
        return new Promise((resolve, reject) => {
            chrome.debugger.getTargets(targets => {
                const target = targets
                    .filter(target => target.attached && target.tabId === debugee.tabId)
                    .map(target => {
                    return {
                        ...target,
                        targetId: target.id,
                        canAccessOpener: false,
                    };
                });
                target[0] ? resolve(target[0]) : reject(new Error('target not found'));
            });
        });
    }
    _handleCommandResponse(command, result) {
        const error = chrome.runtime.lastError;
        const response = {
            ...command,
            error: error,
            result: result,
        };
        console.log(command);
        this._delaySend(response);
    }
    _handleTargetCommand(command) {
        const response = {
            ...command,
            error: undefined,
            result: {},
        };
        switch (command.method) {
            case 'Target.getBrowserContexts':
                response.result = {
                    browserContextIds: [],
                };
                break;
            case 'Target.setDiscoverTargets':
                response.result = null;
                this._emitTargetCreated();
                break;
            case 'Target.attachToTarget':
                response.result = {
                    sessionId: this._sessionId,
                };
                this._emitTargetAttached();
                break;
            case 'Target.activateTarget':
                response.result = null;
                break;
            case 'Target.closeTarget':
                response.result = {
                    success: true,
                };
                setTimeout(() => this.close(), this.responseDelay);
                break;
        }
        this._delaySend(response);
    }
    _emitTargetCreated() {
        const event = {
            method: 'Target.targetCreated',
            params: {
                targetInfo: this.target,
            },
        };
        this._emit(event);
    }
    _emitTargetAttached() {
        const event = {
            method: 'Target.attachedToTarget',
            params: {
                targetInfo: this.target,
                sessionId: this._sessionId,
                waitingForDebugger: false,
            },
        };
        this._emit(event);
    }
    _emitTargetDetached() {
        const event = {
            method: 'Target.detachedFromTarget',
            params: {
                targetId: this.target.id,
                sessionId: this._sessionId,
            },
        };
        this._emit(event);
    }
    _closeTarget() {
        var _a;
        this._emitTargetDetached();
        (_a = this.onclose) === null || _a === void 0 ? void 0 : _a.call(null);
    }
    _emit(event) {
        var _a;
        (_a = this === null || this === void 0 ? void 0 : this.onmessage) === null || _a === void 0 ? void 0 : _a.call(null, JSON.stringify(event));
    }
    _delaySend(response) {
        setTimeout(() => {
            var _a;
            (_a = this === null || this === void 0 ? void 0 : this.onmessage) === null || _a === void 0 ? void 0 : _a.call(null, JSON.stringify(response));
        }, this.responseDelay);
    }
}
exports.default = ExtensionDebuggerTransport;
//# sourceMappingURL=index.js.map